/*This is just the first pandemic file.  It allows the user to type in a few commands.
  It cans three cities, and allows the user to move between them.
  The user can also quit, print their current location, and print the actions they can perform.
  I'll put Note in for programming notes; they will get removed in later versions.
  Chris Huyck wrote this.  I've not asked the pandemic folks to use their game, so don't 
  distribute it beyond CST 3170.
 */

//Note: you need to import scanner to use it to read from the screen.
import java.util.Scanner;

//There's only one class in this program.  All functions are here, and it runs from
//the main function.
public class pandemic1 {
	//class variables on top
	private static Scanner shellInput;    //These two are for the shell scanner.
	private static boolean shellOpen = false;
	
	//Note static is a funny thing for when you only have one class instance.  Since
	//we're doing everything from the main, the class variables and functions are static.
	//If we allocated the class in the main, then we wouldn't need static.
	
	private static int userLocation = 0;  //This is the user's location that can change.

	//The constant data for cities and connections.
	private static final String[] cities = {"Atlanta","New York","London"};
	private static final int NUMBER_CITIES = 3;
	private static final int[][] connections = {{0,1},{1,2}}; //The connections via offset in 
															  //the cities array.
	private static final int NUMBER_CONNECTIONS = 2;

	//The constants for the commands.
	//Note try to use constants to make your code more readable.
	private static final int QUIT = 0;
	private static final int PRINT_LOCATION = 1;
	private static final int MOVE = 2;
	private static final int PRINT_ACTIONS = 3;
	
	
	//Get the users input and translate it to the constants.  Could do lots more 
	//error handling here.
	private static int processUserInput(String inputString) {
		if (inputString.compareTo("quit") == 0)
			return QUIT;
		else if (inputString.compareTo("location") == 0)
			return PRINT_LOCATION;
		else if (inputString.compareTo("move") == 0)
			return MOVE;
		else if ((inputString.compareTo("actions") == 0) ||
				 (inputString.compareTo("help") == 0))
			return PRINT_ACTIONS;
		else 
			return -1;
	}
	
	//Make sure the scanner is open, then get the user input and make sure it's reasonable.
	//Return the integer command.
	private static int getUserInput() {
		boolean gotReasonableInput = false;
		int processedUserInput = -1;

		//Open up the scanner if it's not already open.
		if (!shellOpen) {
			shellInput = new Scanner(System. in);
			shellOpen = true;
			//todo, add error checking.
		}
		//loop until the user types in a command that is named.  It may not be a valid move.
		while (!gotReasonableInput) {
			String userInput = shellInput.nextLine();
			System.out.println("The user typed:"+ userInput);
			processedUserInput = processUserInput(userInput);
			if (processedUserInput >= 0)
				gotReasonableInput = true;
			else
				System.out.println(userInput + "is not a good command. Try 'actions'.");
				
		}
		
		return processedUserInput;
	}

	private static void echoUserInput(int userInput) {
		System.out.println("The user chose:"+ userInput);
	}
	
	//Look through the connections and see if the city numbers are in them.  If
	//Return whether they are in the list.
	private static boolean citiesAdjacent(int city1,int city2) {
		for (int compareConnection = 0; compareConnection < NUMBER_CONNECTIONS; compareConnection ++) {
			if ((connections[compareConnection][0] == city1) &&
				(connections[compareConnection][1] == city2))
				return true;
			//Need to check both ways A to B and B to A as only one connection is stored.
			else if ((connections[compareConnection][0] == city2) &&
					(connections[compareConnection][1] == city1))
					return true;
			
		}
		return false;
	}
	
	//Ask the user where to move, get the city, and if valid, move the user's location.
	private static void moveUser() {
		System.out.println ("Type where you'd like to move.");
		String userInput = shellInput.nextLine();
		int cityToMoveTo = -1;
		//Look for the city the user specified.
		for (int compareCity = 0; compareCity < NUMBER_CITIES; compareCity ++) {
			if (cities[compareCity].compareTo(userInput) == 0)
				cityToMoveTo = compareCity;
		}
		
		if (cityToMoveTo == -1) {
			System.out.println(userInput + " is not a valid city.");
			return;
		}
		
		//If adjacent move the user, if not print an error.
		if (citiesAdjacent(userLocation,cityToMoveTo)) {
			System.out.println("The user has moved from " +
					cities[userLocation] + " to " + 
					cities[cityToMoveTo] + ".");
			userLocation = cityToMoveTo;
		}
		else System.out.println ("You can't move to " + userInput + ".");
		
	}
	
	//Print out all possible user actions.
	private static void printActions() {
		System.out.println ("Type in on the terminal with the following followed by no spaces finish with return.");
		System.out.println ("quit");
		System.out.println ("location");
		System.out.println ("move");
		System.out.println ("actions");
	}
	
	//Handle the user's commands.
	private static boolean processUserInput(int userInput) {
		echoUserInput(userInput);
		
		if (userInput == QUIT) 
			return true;
		else if (userInput == PRINT_LOCATION)
			System.out.println ("The user is in " + cities[userLocation]);
		else if (userInput == MOVE)
			moveUser();
		else if (userInput == PRINT_ACTIONS)
			printActions();
		return false;

	}
	
	//The main function of the program.  Enter and exit from here.
	//It is a simple getInput processInput loop until the game is over.  
	public static void main(String[] args) {
		boolean gameDone = false;

		System.out.println("Hello Pandemic Tester");
		
		while (!gameDone) {
			int userInput = getUserInput();
		
			gameDone = processUserInput(userInput);
		}
		
		System.out.println("Goodbye Pandemic Tester");
	}

}
